@@ -50,6 +50,7 @@ from pyqode.python.modes import (
 from pyqode.core.managers import backend
 from qtpy.QtGui import QFontMetrics
 from qtpy.QtWidgets import QPlainTextEdit
+from qtpy.QtCore import QProcess
 
 
 class pyqode_manager(base_extension):
@@ -141,20 +142,32 @@ class pyqode_manager(base_extension):
 			for editor in self.auto_editor.values():
 				self.event_unregister_editor(editor)
 		self.event_unregister_editor(item._script_widget)
-
-	def event_run_experiment(self, fullscreen):
-
+		
+	def _suspend(self):
+		
 		self._auto_backend_restart = False
 		for editor in self._editors:
 			editor.backend.suspend()
 		self._auto_backend_restart = True
-
-	def event_end_experiment(self, ret_val):
-
+		
+	def _resume(self):
+		
 		for editor in self._editors:
 			editor.backend.resume()
 			self._register_backend_process(editor)
 
+	def event_run_experiment(self, fullscreen):
+
+		self._suspend()
+
+	def event_end_experiment(self, ret_val):
+		
+		self._resume()
+
+	def event_run_experiment_canceled(self):
+		
+		self._resume()
+
 	def event_prepare_purge_unused_items(self):
 
 		for name in self.item_store.unused():
@@ -220,7 +233,7 @@ class pyqode_manager(base_extension):
 		# This function is defined here so to keep the editor in the scope.
 		# This is necessary in case the backend crashes and it needs to be
 		# restarted.
-		def auto_restart():
+		def on_error():
 			if editor._closed or not self._auto_backend_restart:
 				return
 			oslogger.warning(
@@ -230,12 +243,19 @@ class pyqode_manager(base_extension):
 				del editor.backend.SHARE_COUNT[editor.backend._share_id]
 			editor.backend._process = None
 			editor.backend.resume()
+			editor.backend._process.error.connect(on_error)
+			editor.backend._process.finished.connect(on_finished)
 			self._register_backend_process(editor)
+		def on_finished(exit_status):
+			if exit_status == QProcess.CrashExit:
+				oslogger.warning('backend crashed (exit status: {})'.format(exit_status))
+				on_error()
 		try:
-			editor.backend._process.error.connect(auto_restart)
+			editor.backend._process.error.connect(on_error)
+			editor.backend._process.finished.connect(on_finished)
 			self._register_backend_process(editor)
 		except RuntimeError:
-			auto_restart()
+			on_error()
 		oslogger.debug(u'registering {}'.format(editor))
 
 	def event_unregister_editor(self, editor):
